/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.maven.plugin.obfuscation;

import de.schlichtherle.truezip.file.TFileInputStream;
import de.schlichtherle.truezip.file.TFileOutputStream;
import java.io.*;
import javax.annotation.concurrent.NotThreadSafe;
import org.slf4j.Logger;

/**
 * @author Christian Schlichtherle
 */
@NotThreadSafe
abstract class Pass implements Runnable {

    final Processor ctx;

    Pass(final Processor ctx) {
        assert null != ctx;
        this.ctx = ctx;
    }

    final Logger logger() { return ctx.logger(); }

    final Logger logger(Node node) { return ctx.logger(node.path()); }

    final boolean obfuscateAll() { return ctx.obfuscateAll(); }

    final String format(String stage, int index) {
        return ctx.methodName(stage, index);
    }

    @Override
    public void run() { scan(new Node("", ctx.directory())); }

    private void scan(final Node node) {
        final File file = node.file();
        if (file.isDirectory()) {
            final String[] members = file.list();
            if (null != members)
                for (final String member : members)
                    scan(new Node(node, member));
            else
                logger(node).error("Cannot list directory.");
        } else if (file.isFile() && file.getName().endsWith(".class")) {
            process(node);
        } else {
            logger(node).trace("Skipping resource file.");
        }
    }

    abstract void process(final Node clazz);

    final byte[] read(final Node clazz) throws IOException {
        final File file = clazz.file();
        final byte[] code;
        {
            final long l = file.length();
            final int mb = ctx.maxBytes();
            if (l > mb)
                throw new IOException(
                        String.format("%s (%,d bytes exceeds max %,d bytes)", file, l, mb));
            code = new byte[(int) l];
        }
        {
            final InputStream in = new TFileInputStream(file);
            try { new DataInputStream(in).readFully(code); }
            finally { in.close(); }
        }
        return code;
    }

    final void write(final Node clazz, final byte[] code) throws IOException {
        final File file = clazz.file();
        final OutputStream out = new TFileOutputStream(file);
        try { out.write(code); }
        finally { out.close(); }
    }
}
